WITH Calendar;
WITH Text_IO;
WITH My_Int_IO;
PACKAGE BODY Dates IS

-- body for package to represent calendar dates
-- in a form convenient for reading and displaying.

  PACKAGE Month_IO IS 
    NEW Text_IO.Enumeration_IO(Enum => Months);

  FUNCTION Today RETURN Date IS
 
  -- Finds today's date and returns it as a record of type Date
  -- Today's date is gotten from PACKAGE Calendar
 
    Right_Now  : Calendar.Time;           -- holds internal clock value
    Temp       : Date; 
   
  BEGIN -- Today
 
    -- Get the current time value from the computer's clock
    Right_Now := Calendar.Clock;
 
    -- Extract the current month, day, and year from the time value 
    Temp.Month := Months'Val(Calendar.Month(Right_Now)- 1); 
    Temp.Day := Calendar.Day  (Right_Now);
    Temp.Year := Calendar.Year (Date => Right_Now);
 
    RETURN Temp;

  END Today;

  FUNCTION Date_Of(Year : Year_Number; 
                   Month : Months; 
                   Day : Day_Number) RETURN Date IS

  -- constructs a date given year, month, and day.
  -- Pre: Year, Month, and Day are initialized
  -- Post: returns the corresponding Date record, or raises Date_Error
  --       if the date would be invalid

    Temp: Calendar.Time;

  BEGIN -- Date_Of

    Temp := Calendar.Time_Of(Year=>Year, Month=>Months'Pos(Month)+1, Day=>Day);
    -- assert: M, D, and Y form a sensible date if Time_error not raised

    RETURN  (Month => Month, Year => Year, Day => Day);
    -- assert: a valid date is returned
  
  EXCEPTION
  
    WHEN Calendar.Time_Error =>
      RAISE Date_Error;

  END Date_Of;

  FUNCTION Year (D: Date) RETURN Year_Number IS
  BEGIN
    RETURN D.Year;
  END Year;

  FUNCTION Month (D: Date) RETURN Months IS
  BEGIN
    RETURN D.Month;
  END Month;

  FUNCTION Day (D: Date) RETURN Day_Number IS
  BEGIN
    RETURN D.Day;
  END Day;

  PROCEDURE Get(File: IN Text_IO.File_Type; Item: OUT Date) IS
  -- gets a date in the form MMM DD YYYY
  -- Post: returns a date in Item

    M:    Months;
    D:    Calendar.Day_Number;
    Y:    Calendar.Year_Number;

  BEGIN

    Month_IO.Get (File => File, Item => M);
    My_Int_IO.Get(File => File, Item => D);
    My_Int_IO.Get(File => File, Item => Y);
    -- assert: M, D, and Y are well-formed and in range
    --         otherwise one of the Get's would raise an exception

    Item := Date_Of (Month => M, Year => Y, Day => D);
    -- assert: Item is a valid date if Date_Error not raised

  EXCEPTION

    WHEN Text_IO.Data_Error =>
      RAISE Date_Error;
    WHEN Constraint_Error =>
      RAISE Date_Error;
    WHEN Date_Error =>
      RAISE Date_Error;

  END Get;

  PROCEDURE WriteShort(File: IN Text_IO.File_Type; Item: IN Date) IS

    Last2Digits : Natural;

  -- writes a date in dd MMM yy form to a file
  -- Pre: Item is assigned a value
  -- Post: Writes a date in dd MMM yy form

  BEGIN

    Last2Digits := Item.Year MOD 100;

    IF Item.Day < 10 THEN
      Text_IO.Put(File => File, Item => '0');
    END IF;
    My_Int_IO.Put(File => File, Item => Item.Day, Width => 1);
    Text_IO.Put(File => File, Item => ' '); 
    Month_IO.Put (File => File, Item => Item.Month, Width => 1);
    Text_IO.Put(File => File, Item => ' ');
    IF Last2Digits < 10 THEN
      Text_IO.Put(File => File, Item => '0');
    END IF;
    My_Int_IO.Put(File => File, Item => Last2Digits, Width => 1);

  END WriteShort;

  PROCEDURE WriteFull(File: IN Text_IO.File_Type; Item: IN Date) IS

  -- writes a date in full Monthname dd, yyyy form
  -- Pre: Item is assigned a value
  -- Post: Writes a date in Monthname dd, yyyy form

  BEGIN

    CASE Item.Month IS
      WHEN Jan =>
        Text_IO.Put(File => File, Item => "January");
      WHEN Feb =>
        Text_IO.Put(File => File, Item => "February");
      WHEN Mar =>
        Text_IO.Put(File => File, Item => "March");
      WHEN Apr =>
        Text_IO.Put(File => File, Item => "April");
      WHEN May =>
        Text_IO.Put(File => File, Item => "May");
      WHEN Jun =>
        Text_IO.Put(File => File, Item => "June");
      WHEN Jul =>
        Text_IO.Put(File => File, Item => "July");
      WHEN Aug =>
        Text_IO.Put(File => File, Item => "August");
      WHEN Sep =>
        Text_IO.Put(File => File, Item => "September");
      WHEN Oct =>
        Text_IO.Put(File => File, Item => "October");
      WHEN Nov =>
        Text_IO.Put(File => File, Item => "November");
      WHEN Dec =>
        Text_IO.Put(File => File, Item => "December");
    END CASE;

    Text_IO.Put(File => File, Item => ' ');
    My_Int_IO.Put(File => File, Item => Item.Day, Width => 1);
    Text_IO.Put(File => File, Item => ", "); 
    My_Int_IO.Put(File => File, Item => Item.Year, Width => 1);

  END WriteFull;

  PROCEDURE WriteNumeric(File: IN Text_IO.File_Type; Item: IN Date) IS

  -- writes a date in mm/dd/yy form to a file
  -- Pre: Item is assigned a value
  -- Post: Writes a date in mm/dd/yy form

    Last2Digits : Natural;

  BEGIN

    Last2Digits := Item.Year MOD 100;

    My_Int_IO.Put (File => File, Item => Months'Pos(Item.Month)+1, Width => 1);
    Text_IO.Put(File => File, Item => '/');
    My_Int_IO.Put(File => File, Item => Item.Day, Width => 1);
    Text_IO.Put(File => File, Item => '/'); 
    IF Last2Digits < 10 THEN
      Text_IO.Put(File => File, Item => '0');
    END IF;
    My_Int_IO.Put(File => File, Item => Last2Digits, Width => 1);

  END WriteNumeric;

  PROCEDURE WriteJulian(File: IN Text_IO.File_Type; Item: IN Date) IS

  -- writes a date in yyddd form to a file
  -- Pre: Item is assigned a value
  -- Post: Writes a date in yyddd form

    Last2Digits : Natural;
    JulianDay   : Natural;

    TYPE NumberOfDays IS ARRAY (Months) OF Positive;
    NonLeapDays : CONSTANT NumberOfDays :=
      (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
    LeapDays : CONSTANT NumberOfDays :=
      (31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);

  BEGIN

    Last2Digits := Item.Year MOD 100;
    JulianDay := Item.Day;

    IF Item.Month > Months'First THEN
      IF (Item.Year MOD 400 = 0)
         OR ((Item.Year MOD 4 = 0) AND (Item.Year MOD 100 /= 0)) THEN
        FOR Mon IN Months'First..Months'Pred(Item.Month) LOOP
          JulianDay := JulianDay + LeapDays(Mon);
        END LOOP;
      ELSE
        FOR Mon IN Months'First..Months'Pred(Item.Month) LOOP
          JulianDay := JulianDay + NonLeapDays(Mon);
        END LOOP;
      END IF;
    END IF;

    IF Last2Digits < 10 THEN
      Text_IO.Put(File => File, Item => "0");
    END IF;
    My_Int_IO.Put(File => File, Item => Last2Digits, Width => 1);

    IF JulianDay < 10 THEN
      Text_IO.Put(File => File, Item => "00");
    ELSIF JulianDay < 100 THEN
      Text_IO.Put(File => File, Item => "0");
    END IF;
    My_Int_IO.Put(File => File, Item => JulianDay, Width => 1);

  END WriteJulian;

  PROCEDURE Put(File: IN Text_IO.File_Type; 
                Item: IN Date; Format: IN Formats) IS
  BEGIN
    CASE Format IS
      WHEN Short =>
        WriteShort(File => File, Item => Item);
      WHEN Full =>
        WriteFull(File => File, Item => Item);
      WHEN Numeric =>
        WriteNumeric(File => File, Item => Item);
      WHEN Julian =>
        WriteJulian(File => File, Item => Item);
    END CASE;
  END Put;

  PROCEDURE Get(Item: OUT Date) IS
  BEGIN
    Get(File => Text_IO.Standard_Input, Item => Item);
  END Get;

  PROCEDURE Put(Item: IN Date; Format: IN Formats) IS
  BEGIN
    Put(File => Text_IO.Standard_Output, Item => Item, Format => Format);
  END Put;

END Dates;
