WITH Text_IO;
PACKAGE VStrings IS

  -- Specification for ADT to handle strings of variable length 1-80 chars.
 
  MaxLength         : CONSTANT Integer := 80;
  SUBTYPE Index IS Integer RANGE 0 .. MaxLength;

  TYPE    VString  IS PRIVATE;
 
  -- exceptions

  StringOverflow   : EXCEPTION;
  EmptyString      : EXCEPTION;
  InvalidArguments : EXCEPTION;

  -- operators

  -- constructors

  FUNCTION MakeVString(S : String) RETURN VString;
  FUNCTION MakeVString(C : Character) RETURN VString;
  FUNCTION EmptyVString RETURN VString;
  -- construct a VString
  -- Pre: parameters are defined
  -- Post: returns a VString as desired. 
  --   Raises StringOverflow if S is longer than MaxLength characters

  -- selectors

  FUNCTION Length(S : VString) RETURN Index;
  FUNCTION Value(S : VString) RETURN String;
  FUNCTION Head(S : VString) RETURN Character;
  -- return the length, the string part, and the first character respectively
  -- Pre: parameters are defined
  -- Post: returns the desired value
  --   Head raises EmptyString if S is empty
 
  -- inquiry

  FUNCTION IsEmpty(S : VString) RETURN Boolean;
  -- tests whether the string is 0 characters long
  -- Pre: S is defined
  -- Post: returns True if S is empty, False otherwise
  -- concatenation

  FUNCTION "&" (S1, S2 : VString) RETURN VString;
  FUNCTION "&" (S1 : VString; C : Character) RETURN VString;
  FUNCTION "&" (C : Character; S1 : VString) RETURN VString;
  FUNCTION "&" (S1 : VString; S : String) RETURN VString;
  FUNCTION "&" (S : string; S1 : VString) RETURN VString;
  -- these five operators support concatenation of VStrings
  -- Pre: all parameters are defined
  -- Post: each operator returns the concatenation of its arguments,
  --   or raises StringOverflow if the result would be longer than
  --   MaxLength characters
 
  -- lexical comparison

  FUNCTION "<" (S1, S2 : VString) RETURN Boolean;
  FUNCTION "<=" (S1, S2 : VString) RETURN Boolean;
  FUNCTION ">" (S1, S2 : VString) RETURN Boolean;
  FUNCTION ">=" (S1, S2 : VString) RETURN Boolean;
  -- these four operators compare VStrings, lexically
  -- Pre: S1 and S2 are defined
  -- Post: carries out the desired comparison, returning True or False
 
  -- search

  FUNCTION Locate(Sub : VString; Within : VString) RETURN Index;
  FUNCTION Locate(Sub : String; Within : VString) RETURN Index;
  FUNCTION Locate(C : Character; Within : VString) RETURN Index;
  -- these three operators search for Sub in Within
  -- Pre: Sub and Within are defined
  -- Post: returns the index of the first character of Sub in Within;
  --   returns 0 if Sub is not present in Within
 
  FUNCTION Tail(S : VString) RETURN VString;
  -- returns a string similar to S but with the first character removed
  -- Pre: S is defined
  -- Post: returns a string like S but with the first character removed;
  --   raises EmptyString if S is empty

  FUNCTION SubString(S : VString; Start, Size : Index) RETURN VString;
  -- returns a string containing Size characters, whose value is the
  -- Size-character substring starting at position Start in S.
  -- Pre: parameters are defined
  -- Post: raises InvalidParameters if Start > Length(S),
  --   or Size > Length(S), or Start + Size -1 > Length(S)
 
  -- input/output

  PROCEDURE Get_Line(Item : OUT VString);
  PROCEDURE Put(Item : VString);

  PROCEDURE Get_Line(File: Text_IO.File_Type; Item : OUT VString);
  PROCEDURE Put(File: Text_IO.File_Type; Item : VString);
 
PRIVATE

  TYPE VString IS RECORD
    CurrentLength : Index := 0;
    StringPart : String(1 .. MaxLength) := (OTHERS => ASCII.NUL);
  END RECORD;

END VStrings;
