WITH My_Flt_IO;
WITH Text_IO;
WITH RobustInput;
PACKAGE BODY Geometry IS

-- Defines an abstract data type for a geometric figure.  Operators
-- Operators include constructors for rectangles, circles, and squares,
-- selectors for area and perimeter, and input/output operations
  
  PACKAGE FigKind_IO IS NEW Text_IO.Enumeration_IO (Enum => FigKind);

  Pi : CONSTANT Float := 3.14159;
  MaxSize : CONSTANT Float := 1_000_000.00;

-- Data type Figure is declared in the package specification

-- Local procedure ReadShape is used only within the package, 
-- therefore it is not exported (does not appear in the spec).

  PROCEDURE ReadShape (Item : OUT FigKind) IS 

    TempItem: FigKind;

  BEGIN -- ReadShape
            
    LOOP
      BEGIN
        Text_IO.Put(Item => "Enter a shape: rect, circle, square > ");
        FigKind_IO.Get(Item => TempItem);
        Item := TempItem;
        EXIT;
      EXCEPTION
        WHEN Text_IO.Data_Error =>
          Text_IO.Put ("Value not a valid shape. Please try again.");
          Text_IO.New_Line;
          Text_IO.Skip_Line;
      END;
    END LOOP;
    -- assert: Item is rect, circle, or square

  END ReadShape;

  -- local functions, not exported in the spec

  FUNCTION ComputePerimeter (OneFig : Figure) RETURN NonNegFloat IS
  
  -- Computes Perimeter of OneFig.
  -- Pre : The discriminant and characteristics of OneFig are defined.
  -- Post: Returns Perimeter

  BEGIN -- ComputePerimeter   
    
    CASE OneFig.FigShape IS
      WHEN Rect =>
        RETURN 2.0 * (OneFig.Width + OneFig.Height);
      WHEN Square =>
        RETURN 4.0 * OneFig.Side;
      WHEN Circle =>
        RETURN 2.0 * Pi * OneFig.Radius;
    END CASE;
    
  END ComputePerimeter;


  FUNCTION ComputeArea (OneFig : Figure) RETURN NonNegFloat IS
  
  -- Computes Area of OneFig.
  -- Pre : The discriminant and characteristics of OneFig are defined.
  -- Post: Returns Area of OneFig.
    
  BEGIN -- ComputeArea   
    
    CASE OneFig.FigShape IS
      WHEN Rect =>
        RETURN OneFig.Width * OneFig.Height;
      WHEN Square =>
        RETURN OneFig.Side ** 2;
      WHEN Circle =>
        RETURN Pi * OneFig.Radius ** 2 ;
    END CASE;

  END ComputeArea;

-- Exported Operators   

  FUNCTION MakeRect (Width, Height : NonNegFloat) RETURN Figure IS

  -- constructor: creates a rectangular figure from its height and width
  -- Pre : Width and Height are defined
  -- Post: returns a figure of type Rect

    Temp : Figure(FigShape => Rect);

  BEGIN -- MakeRect

    Temp.Height    := Height;
    Temp.Width     := Width; 
    Temp.Area      := ComputeArea(Temp);
    Temp.Perimeter := ComputePerimeter(Temp);

    RETURN Temp;

  END MakeRect;


  FUNCTION MakeCircle (Radius : NonNegFloat) RETURN Figure IS

  -- constructor: creates a circular figure from its radius
  -- Pre : Radius is defined
  -- Post: returns a figure of type Circle

    Temp: Figure (FigShape => Circle);

  BEGIN -- MakeCircle

    Temp.Radius    := Radius;
    Temp.Area      := ComputeArea(Temp);
    Temp.Perimeter := ComputePerimeter(Temp);

    RETURN Temp;

  END MakeCircle;


  FUNCTION MakeSquare (Side : NonNegFloat) RETURN Figure IS

  -- constructor: creates a square figure from its side
  -- Pre : Side is defined
  -- Post: returns a figure of type Square

    Temp: Figure (FigShape => Square);

  BEGIN -- MakeSquare

    Temp.Side      := Side  ;
    Temp.Area      := ComputeArea(Temp);
    Temp.Perimeter := ComputePerimeter(Temp);

    RETURN Temp;

  END MakeSquare;

  
  FUNCTION Perimeter (OneFig : Figure) RETURN NonNegFloat IS
  
  -- selector: returns Perimeter of OneFig.
  -- Pre : The discriminant field and characteristics of OneFig are defined.
  -- Post: Returns Perimeter

  BEGIN -- Perimeter   

    RETURN OneFig.Perimeter;

  END Perimeter;


  FUNCTION Area (OneFig : Figure) RETURN NonNegFloat IS
  
  -- selector: returns Area of OneFig.
  -- Pre : The discriminant field and characteristics of OneFig are defined.
  -- Post: Returns Area

  BEGIN -- Area   

    RETURN OneFig.Area;

  END Area;


  PROCEDURE ReadFigure (OneFig : OUT Figure) IS

    Shape : FigKind;
    Height : NonNegFloat;
    Width  : NonNegFloat;
    Side   : NonNegFloat;
    Radius : NonNegFloat;

  BEGIN  -- ReadFigure   

    -- Read the shape character and define the discriminant
    ReadShape(Shape);

    -- Select the proper variant and read pertinent data   
    CASE Shape IS
      WHEN Rect =>
        Text_IO.Put(Item => "Enter width.");
        Text_IO.New_Line;
        RobustInput.Get(Item => Width, MinVal => 0.0, MaxVal => MaxSize);   
        Text_IO.Put(Item => "Enter height.");
        Text_IO.New_Line;
        RobustInput.Get(Item => Height, MinVal => 0.0, MaxVal => MaxSize);
        OneFig := MakeRect(Width, Height);
   
      WHEN Square    =>
        Text_IO.Put(Item => "Enter length of side.");
        Text_IO.New_Line;
        RobustInput.Get(Item => Side, MinVal => 0.0, MaxVal => MaxSize);   
        OneFig := MakeSquare(Side);

      WHEN Circle    =>
        Text_IO.Put(Item => "Enter circle radius.");
        Text_IO.New_Line;
        RobustInput.Get(Item => Radius, MinVal => 0.0, MaxVal => MaxSize);
        OneFig := MakeCircle(Radius);

    END CASE;

  END ReadFigure;
    

  PROCEDURE DisplayFigure (OneFig : IN Figure) IS
  
  -- Displays the characteristics of OneFig.
  -- Pre : All fields of OneFig are defined.
  -- Post: Displays each field of OneFig.
    
  BEGIN -- DisplayFigure   
    
    -- Display shape and characteristics   
    Text_IO.Put(Item => "Figure shape: ");
    FigKind_IO.Put(Item => OneFig.FigShape, Width => 1);
    Text_IO.New_Line;

    CASE OneFig.FigShape IS
      WHEN Rect =>
        Text_IO.Put(Item => "height = ");
        My_Flt_IO.Put(Item => OneFig.Height, Fore=>1, Aft=>2, Exp=>0);
        Text_IO.Put(Item => "; width = ");
        My_Flt_IO.Put(Item => OneFig.Width, Fore=>1, Aft=>2, Exp=>0);

      WHEN Square =>
        Text_IO.Put(Item => "side = ");
        My_Flt_IO.Put(Item => OneFig.Side, Fore=>1, Aft=>2, Exp=>0);
        
      WHEN Circle =>
        Text_IO.Put(Item => "radius = ");
        My_Flt_IO.Put(Item => OneFig.Radius, Fore=>1, Aft=>2, Exp=>0);
        
    END CASE;

    Text_IO.Put(Item => "; perimeter = ");
    My_Flt_IO.Put(Item => OneFig.Perimeter, Fore=>1, Aft=>2, Exp=>0);
    Text_IO.Put(Item => "; area = ");
    My_Flt_IO.Put(Item => OneFig.Area, Fore=>1, Aft=>2, Exp=>0);
    Text_IO.New_Line;

  END DisplayFigure;

END Geometry;
