PACKAGE MetricSystem IS
 
--  This is the specification of the package MetricSystem.
--  Type definition
 
  TYPE    Metric(Mass, Length, Time : Integer) IS PRIVATE;
 
  SUBTYPE Scalar   IS Metric(0, 0, 0);
 
  SUBTYPE Accel    IS Metric(0, 1, -2);
  SUBTYPE Area     IS Metric(0, 2, 0);
  SUBTYPE Length   IS Metric(0, 1, 0);
  SUBTYPE Distance IS Metric(0, 1, 0);
  SUBTYPE Mass     IS Metric(1, 0, 0);
  SUBTYPE Time     IS Metric(0, 0, 1);
  SUBTYPE Velocity IS Metric(0, 1, -1);
  SUBTYPE Volume   IS Metric(0, 3, 0);
  
 
  Gram       : CONSTANT Metric;
  METER      : CONSTANT Metric;
  SEC        : CONSTANT Metric;
  Square_M   : CONSTANT Metric;
  Cubic_M    : CONSTANT Metric;
  M_per_Sec  : CONSTANT Metric;
  M_per_Sec2 : CONSTANT Metric;
 
  -- dyadic operator to produce a dimensioned quantity from a scalar
  -- Value
 
  FUNCTION "*" (Left : Float; Right : Metric) RETURN Metric;
 
  --  The monadic operators are passed a single variable, Right, of
  --  type Metric.  Right can be of any subtype of Metric.  The
  --  result is of the same subtype.
 
  FUNCTION "+" (Right : Metric) RETURN Metric;
  FUNCTION "-" (Right : Metric) RETURN Metric;
  FUNCTION "abs" (Right : Metric) RETURN Metric;
  
 
  --  The dyadic operators are passed two variables, Left and Right, of
  --  any subtype of Metric (not necessarily the same).  The
  --  result type depends on the operation and subtypes of the
  --  variables passed.
 
  
  --  "+" and "-" require two variables of the same subtype,
  --  they return a variable of the same type passed
 
  FUNCTION "+" (Left, Right : Metric) RETURN Metric;
  FUNCTION "-" (Left, Right : Metric) RETURN Metric;
  
 
  --  "*" and "/" require variables of any subtype
  --  of Metric.  The subtype of the variable returned depends on
  --  the types passed and how the operation combines the units.
 
  FUNCTION "*" (Left, Right : Metric) RETURN Metric;
  FUNCTION "/" (Left, Right : Metric) RETURN Metric;
  
 
  --  "<", "<=", ">", ">=" all require two variables of the same
  --  subtype and return a boolean variable
 
  FUNCTION "<"  (Left, Right : Metric) RETURN Boolean;
  FUNCTION "<=" (Left, Right : Metric) RETURN Boolean;
  FUNCTION ">"  (Left, Right : Metric) RETURN Boolean;
  FUNCTION ">=" (Left, Right : Metric) RETURN Boolean;
 
  --  Value returns the Float Value of a variable of
  --  a subtype of Metric
 
  FUNCTION Value(Left : Metric) RETURN Float;
 
  Dimension_Error : EXCEPTION;
 
PRIVATE
 
  TYPE Metric(Mass, Length, Time : Integer) IS RECORD
    Value : Float := 0.0;
  END RECORD;
 
  Gram       : CONSTANT Metric := (1, 0, 0, 1.0);
  METER      : CONSTANT Metric := (0, 1, 0, 1.0);
  SEC        : CONSTANT Metric := (0, 0, 1, 1.0);
  Square_M   : CONSTANT Metric := (0, 2, 0, 1.0);
  Cubic_M    : CONSTANT Metric := (0, 3, 0, 1.0);
  M_per_Sec  : CONSTANT Metric := (0, 1, -1, 1.0);
  M_per_Sec2 : CONSTANT Metric := (0, 1, -2, 1.0);
 
END MetricSystem;
