WITH Dates;
WITH MonthlyTotals;
WITH AnnualTotals;
PACKAGE SalesTable IS

-- Package Specification for abstract data type for a table of sales
-- data organized by year and month.  The operations performed
-- include: reading the sales data, updating sales, displaying the data,
-- and tabulating sums by year or month.

  -- Data Types

  TYPE SalesArray IS ARRAY (AnnualTotals.YearRange, Dates.Months) OF Float;
  TYPE SalesRecord IS RECORD  -- all fields initialized by default
    FirstYear : AnnualTotals.YearRange := AnnualTotals.YearRange'First;
    LastYear : AnnualTotals.YearRange := AnnualTotals.YearRange'Last;
    Volume    : SalesArray := (OTHERS => (OTHERS => 0.0));
  END RECORD; -- SalesRecord   


  -- operators  

  PROCEDURE EnterSales (Sales : OUT SalesRecord);
  
  -- Reads the sales data into the record Sales.
  -- Pre : None
  -- Post: The fields FirstYear and LastYear are defined and
  --      sales data are read into array Volume for each month of
  --      years FirstYear..LastYear.
   

  PROCEDURE UpdateSales (Sales : IN OUT SalesRecord);
  
  -- Reads the sales data into the record Sales.
  -- Pre : Record Sales is defined
  -- Post: Reads year, month, and value for each update and
  --       stores the new value in array Sales.Volume
   

  PROCEDURE DisplaySales (Sales : IN SalesRecord);
  
  -- Displays the sales data.
  -- Pre : Record Sales is defined.
  -- Post: Displays two-dimensional table of sales volume by
  --      month for years FirstYear..LastYear.
   

  PROCEDURE TabYear (Sales : IN SalesRecord;
                    SumByYear : OUT AnnualTotals.YearSums);
  
  -- Tabulates sales totals by year.  Sums are stored in
  -- array SumByYear.
  -- Pre : Record Sales is defined.
  -- Post: Elements SumByYear(FirstYear-MinYear) through
  --      SumByYear(LastYear-MinYear) are tabulated.
   

  PROCEDURE TabMonth (Sales : IN SalesRecord;
                     SumByMonth : OUT MonthlyTotals.MonthSums);
  
  -- Tabulates sales totals by month.  Sums are stored in
  -- array SumByMonth.
  -- Pre : Record Sales is defined.
  -- Post: Elements SumByMonth(Jan..Dec) are tabulated for
  --      the years FirstYear..LastYear.
   

END SalesTable;

