WITH Text_IO;
WITH My_Int_IO;
WITH My_Flt_IO;
WITH AnnualTotals;
WITH MonthlyTotals;
WITH RobustInput;
WITH Dates;
WITH Month_IO;
PACKAGE BODY SalesTable IS

 -- Package body for abstract data type for a table of sales
 -- data organized by year and month.  The operations performed
 -- include: reading the sales data, displaying the data,
 -- tabulating sums by year or month.
  
  -- Operators  

  PROCEDURE EnterSales (Sales : OUT SalesRecord) IS
  
  -- Reads the sales data into the record Sales.
  -- Pre : None
  -- Post: The fields FirstYear and LastYear are defined and
  --      sales data are read into array Volume for each month of
  --      years FirstYear..LastYear.

    TempFirst : AnnualTotals.YearRange; 
    TempLast  : AnnualTotals.YearRange; 

  BEGIN -- EnterSales   
    -- Enter first and last years of sales data.   
    Text_IO.Put(Item => "Enter first year of sales data: ");  
    Text_IO.New_Line;
    RobustInput.Get(MinVal=>AnnualTotals.MinYear, 
                    MaxVal=>AnnualTotals.MaxYear, Item=>TempFirst);
    Sales.FirstYear := TempFirst;

    Text_IO.Put(Item => "Enter last  year of sales data: ");  
    Text_IO.New_Line;
    RobustInput.Get(MinVal=>AnnualTotals.MinYear, 
                    MaxVal=>AnnualTotals.MaxYear, Item=>TempLast);
    Sales.LastYear := TempLast;

    -- Enter table data.   
    FOR CurYear IN TempFirst .. TempLast LOOP
      Text_IO.Put(Item => "For year ");
      My_Int_IO.Put(Item => CurYear, Width=>4);
      Text_IO.New_Line;
      Text_IO.Put(Item => "Enter sales amount (Float) for each month or 0.0");
      Text_IO.New_Line;
      FOR CurMonth IN Dates.Months LOOP
        Month_IO.Put(Item=>CurMonth, Width=>3);
        Text_IO.Put(Item => " $");
        My_Flt_IO.Get(Item => Sales.Volume(CurYear, CurMonth));
      END LOOP;
      Text_IO.New_Line;
    END LOOP;

  END EnterSales;

  PROCEDURE UpdateSales (Sales : IN OUT SalesRecord) IS

  -- Reads the sales data into the record Sales.
  -- Pre : Record Sales is defined
  -- Post: Reads year, month, and value for each update and
  --       stores the new value in array Sales.Volume

    NewYear   : AnnualTotals.YearRange;
    NewMonth  : Dates.Months;
    NewAmount : Float;
    MoreChar  : Character;

  BEGIN

    Text_IO.Put(Item => "Enter year, month, and amount for each update.");
    Text_IO.New_Line;
    LOOP
      Text_IO.Put(Item => "For year - ");
      My_Int_IO.Get(Item => NewYear);
      Text_IO.Put(Item => "For Month ");
      Month_IO.Get(Item => NewMonth);
      Text_IO.Put(Item => "Enter Amount $");
      My_Flt_IO.Get(Item => NewAmount);
      Sales.Volume(NewYear, NewMonth) := NewAmount;
      Text_IO.Put(Item => "Any more changes? Enter Y (Yes) or N (No) > ");
      Text_IO.Get(Item => MoreChar);
      EXIT WHEN (MoreChar = 'n') OR (MoreChar = 'N');
    END LOOP;

  END UpdateSales;


  PROCEDURE ShowHalf (Sales : IN SalesRecord;
                      FirstMonth, LastMonth : IN Dates.Months) IS
  
  -- Displays the sales amounts by year for each of the months from
  -- FirstMonth to LastMonth.
  -- Pre : Array Sales and FirstMonth and LastMonth are defined.
  -- Post: Displays sales volumes in a table whose rows are
  --       FirstYear..LastYear and whose columns are FirstMonth..
  --       LastMont.
    
  BEGIN -- ShowHalf   
    -- Print table heading for 6 months of each year.   
    Text_IO.Put(Item => "Year       ");
    FOR CurMonth IN FirstMonth .. LastMonth LOOP
      Month_IO.Put(Item=>CurMonth, Width=>11);      -- Print month names   
    END LOOP;
    Text_IO.New_Line;                               -- End the heading   

    -- Print sales figures for 6 months of each year.   
    FOR CurYear IN Sales.FirstYear .. Sales.LastYear LOOP
      My_Int_IO.Put(Item => CurYear, Width=>4);
      Text_IO.Put(Item => "  ");
      FOR CurMonth IN FirstMonth .. LastMonth LOOP
        My_Flt_IO.Put (Item=>Sales.Volume(CurYear, CurMonth), 
                       Fore=>6, Aft=>2, Exp=>0);
        Text_IO.Put(Item => "  ");
      END LOOP;
      Text_IO.New_Line;
    END LOOP;
  END ShowHalf;


  PROCEDURE DisplaySales (Sales : IN SalesRecord) IS
  
  -- Displays the sales data.
  -- Pre : Record Sales is defined.
  -- Post: Displays two-dimensional table of sales volume by
  --      month for years FirstYear..LastYear.

  -- Due to line length limits,
  -- the first 6 months and last 6 months of each year are shown in
  -- separate tables.  Uses ShowHalf to display each table.
    
  BEGIN -- DisplaySales   
    -- Display first 6 months of array Sales.   
    ShowHalf (Sales, Dates.Jan, Dates.Jun);
    Text_IO.New_Line;

    -- Display last 6 months of array Sales.   
    ShowHalf (Sales, Dates.Jul, Dates.Dec);
  END DisplaySales;


  PROCEDURE TabYear (Sales : IN SalesRecord;
                    SumByYear : OUT AnnualTotals.YearSums) IS
  
  -- Tabulates sales totals by year.  Sums are stored in
  -- array SumByYear.
  -- Pre : Record Sales is defined.
  -- Post: Elements SumByYear(FirstYear-MinYear) through
  --      SumByYear(LastYear-MinYear) are tabulated.

    Sum : Float;                        -- sum for each year   

  BEGIN -- TabYear   
    -- Find each annual total.   
    FOR CurYear IN Sales.FirstYear .. Sales.LastYear LOOP
      -- Accumulate sum for 12 months   
      Sum := 0.0;
      FOR CurMonth IN Dates.Months LOOP
        Sum := Sum + Sales.Volume(CurYear, CurMonth);
      END LOOP;
      SumByYear.Sums(CurYear) := Sum;
    END LOOP;
  END TabYear;


  PROCEDURE TabMonth (Sales : IN SalesRecord;
                     SumByMonth : OUT MonthlyTotals.MonthSums) IS
  
  -- Tabulates sales totals by month.  Sums are stored in
  -- array SumByMonth.
  -- Pre : Record Sales is defined.
  -- Post: Elements SumByMonth(Jan..Dec) are tabulated for
  --      the years FirstYear..LastYear.

  BEGIN -- TabMonth stub   
    Text_IO.Put(Item => "Procedure TabMonth entered.");  
    Text_IO.New_Line;
  END TabMonth;


END SalesTable;
