GENERIC

  TYPE Element IS PRIVATE;     -- assignment and equality predefined
  TYPE KeyType IS PRIVATE;     -- here too

  MinKey: KeyType;
  MaxKey: KeyType;

  -- These generic parameters specify how to insert a key in an
  -- element, retrieve the key from an element, compare elements
  WITH PROCEDURE InsertKey  (El: IN OUT Element; Key: KeyType);
  WITH PROCEDURE ExtractKey (El: Element;        Key: OUT KeyType);
  WITH FUNCTION  Bigger     (Key1, Key2: KeyType) RETURN Boolean;

  -- This parameter specifies what to do with each element during
  -- a traversal of a table;
  WITH PROCEDURE Visit (El: Element);

PACKAGE GenericTables IS

-- Specification of the abstract data type for an ordered table of
-- element records, each containing a key.
-- This version has type definitions to implement the table as a 
-- one-way linked list. The client cannot see or use these types
-- because Table is LIMITED PRIVATE.
  
-- Data Structure   

  TYPE TableType IS LIMITED PRIVATE;

--Exported exception

  UninitializedTable: EXCEPTION;
  -- UnitializedTable will be raised by an operator if its table
  -- parameter has not been initialized

-- Operators   

  PROCEDURE CreateTable (Table : IN OUT TableType);

  -- initializes a Table.
  -- Pre : None
  -- Post: Table is an initialized TableType
     
  FUNCTION SizeOfTable (Table : TableType) RETURN Natural;

  -- Returns the number of elements in a Table
  -- Pre : Table is an initialized TableType
  -- Post: Returns the number of elements in Table

  PROCEDURE Search (Table   : TableType;
		    Target  : KeyType;
		    Success : OUT Boolean);

  --  Searches a Table for Target.
  --  Pre : Table is an initialized TableType
  --  Post: Success is True if Target is found; otherwise,
  --        Success is False.
     
  PROCEDURE Insert (Table   : IN OUT TableType;
		    El      : Element;
		    Success : OUT Boolean);

  --  Inserts item El into a Table.
  --  Pre : Table is an initialized TableType.
  --  Post: Success is True if insertion is performed; Success is False
  --        if insertion is not performed because there is already
  --        an element with the same key as El.
     
  PROCEDURE Delete (Table   : IN OUT TableType;
		    Target  : KeyType;
		    Success : OUT Boolean);

  --  Deletes the element with key Target from a Table.
  --  Pre : Table is an initialized TableType.
  --  Post: Success is True if deletion is performed; Success is False
  --        if deletion is not performed because there is no element
  --        whose key is Target.
     
  PROCEDURE Replace (Table   : IN OUT TableType;
		     El      : Element;
		     Success : OUT Boolean);

  --  Replaces the element of a Table with the same key as
  --  El by the contents of El.
  --  Pre : Table is an initialized TableType.
  --  Post: Success is True if the replacement is performed; Success is
  --        False if there is no element with the same key as El.
     
  PROCEDURE Retrieve (Table   : TableType;
		      Target  : KeyType;
		      El      : OUT Element;
		      Success : OUT Boolean);

  --  Copies the element whose key is Target into El.
  --  Pre : Table is an initialized TableType.
  --  Post: Success is True if the copy is performed; Success is False
  --        if there is no element whose key is Target.
     

  PROCEDURE Traverse (Table : TableType);

  --  Repeatedly calls procedure Visit (a generic parameter) to
  --  process each element of a Table.
  --  Pre : Table is an initialized TableType.
  --  Post: Each element is operated on in turn by procedure Visit.

PRIVATE

  TYPE ListNode;
  TYPE NodePointer IS ACCESS ListNode;
  TYPE ListNode IS RECORD
    Item: Element;
    Link: NodePointer;
  END RECORD;

  TYPE TableType IS RECORD
    Head    : NodePointer;
    NumItems: Natural;
  END RECORD;

END GenericTables;

